var Grupo = require('../models/grupo.js');
  var Participante = require('../models/participante.js');
  var Tematica = require('../models/tematica.js');
  var Usuario = require('../models/usuario.js');
  var waterfall = require('async-waterfall');

  exports.createGrupo = function(req, res) {
    waterfall([
      function (callback){
        Grupo.create({
          nombre : req.body.nombre,
          numero_participantes: 1,
          numero_tematicas: 0
        }, function (err, grupo) {
          if (err){
            res.send(err);
          }
          callback(null, grupo);
        });
      },
      function (grupo, callback){
        Usuario.findOne({
          _id: req.body.usuario_id
        }, function (err, usuario){
          if(err){
            res.send(err);
          }
          callback(null, usuario, grupo);
        });
      },
      function (usuario, grupo, callback){
        // get and return all the calificaciones after you create another
          Participante.create({
            grupo_id: grupo,
            md_perfil_clave: 'QZ',
            ranking: 0,
            participacion: 0,
            usuario_id: usuario
          }, function (err, participante){
            if(err){
              res.send(err);
            }
            callback(null, usuario);
          });
      }
    ], 
    function(err, result){
        res.json({ 
          message: "Grupo creado."
        });
    });
  }

  exports.findAll = function(req, res){
    Grupo.find(function(err, grupos) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(grupos); // return all todos in JSON format
    });
  }

  exports.findAllPublicos = function(req, res){
    Grupo.find({ 
      privado: false
    }, function(err, grupos) {
      if(err){
        return handleError(err);
      }
      res.json(grupos);
    });
  }

  exports.findByQuizzer = function(req, res){
    Participante.find({
      usuario_id: req.params.usuarioId,
      md_perfil_clave: {$ne: 'ZP'}
    },function(err, participantes) {
    // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){res.send(err);}
        var gruposId = [];
        for (var i in participantes) {
          gruposId.push(participantes[i].grupo_id);
        }
        Grupo.find({
          _id: {$in: gruposId}
        },function(err, grupos) {
          if(err){
            return handleError(err);
          }
          res.json(grupos);
        });
    });
  }

  exports.updateGrupo = function(req, res){
    console.log(req.params.id);
    Grupo.findOne({ 
      _id: req.params.id
    }, function (err, grupo) {
      console.log(req.body);
      console.log(grupo);
      if(err){
        return handleError(err);
      }
      if(req.body.url != null){
        grupo.url = req.body.url;
      }
      grupo.nombre = req.body.nombre;      
      grupo.save(function(err) {
        if (err){
          return handleError(err);
        }
        res.json({ 
          message: "Grupo actualizado."
        });
      });
    });
  }

  exports.getGrupo = function(req, res){
    Grupo.findOne({ 
      _id: req.params.id
    }, function(err, grupo) {
      if (err) 
        return handleError(err);
      res.json(grupo);
    });
  }

  exports.findByNombre = function(req, res){
    Grupo.findOne({ 
      nombre: req.params.nombre
    }, function(err, grupo) {
      if (err) 
        return handleError(err);
      res.json(grupo);
    });
  }
  

  exports.findByTematica = function(req, res){
    Tematica.findOne({ 
      _id: req.params.tematicaId
    }, function(err, tematica) {
      Grupo.findOne({ 
        _id: tematica.grupo_id
      }, function(err, grupo) {
        if (err) 
          return handleError(err);
        res.json(grupo);
      });
    });
  }

  // first locates a thread by title, then locates the replies by thread ID.
  exports.deleteGrupo = function(req, res) {
      Grupo.remove({
          _id : req.params.id
      }, function(err, grupo) {
          if (err)
              res.send(err);

          // get and return all the calificaciones after you create another
          Grupo.find(function(err, grupos) {
              if (err)
                  res.send(err)
              res.json(grupos);
          });
      });
  }