var Pregunta = require('../models/pregunta.js');
  var Grupo = require('../models/grupo.js');
  var Participante = require('../models/participante.js');
  var Tematica = require('../models/tematica.js');
  var Respuesta = require('../models/respuesta.js');
  var RespuestaParticipante = require('../models/respuesta_participante.js');
  var Utils = require('../utils/utils.js');
  var waterfall = require('async-waterfall');
  var gcm = require('node-gcm');

  exports.createPregunta = function(req, res) {
      Participante.findOne({
        usuario_id: req.body.usuarioId,
        md_perfil_clave: 'QZ',
        grupo_id: req.body.grupo
      },function(err, participante) {
        Pregunta.create({
            titulo: req.body.titulo,
            enunciado: req.body.enunciado,
            tipo: req.body.tipo,
            numero_respuestas: req.body.numeroRespuestas,
            fecha_alta: Utils.parserDateToStr(req.body.fechaAlta),
            hora_alta: req.body.horaAlta,
            fecha_publicacion: Utils.parserDateToStr(req.body.fechaPublicacion),
            hora_publicacion: req.body.horaPublicacion,
            fecha_limite: Utils.parserDateToStr(req.body.fechaLimite),
            hora_limite: req.body.horaLimite,
            tematica_id: req.body.tematica,
            grupo_id: req.body.grupo,
            quizzer_id: participante
        }, function(err, pregunta) {
            if (err){
              res.send(err);
            }
            Pregunta.find({
              quizzer_id: pregunta.quizzer_id
            })
            .populate({ 
              path: 'quizzer_id',
              match: { usuario_id : req.body.usuarioId}
            })
            .populate(    
              'tematica_id'
              ).exec(function (err, preguntas) {
              // if there is an error retrieving, send the error. nothing after res.send(err) will execute
              if (err){
                  res.send(err);
              }
              var resultadoCrearPregunta = {
                preguntaId: pregunta._id,
                preguntas: preguntas
              };
              res.json(resultadoCrearPregunta); // return all todos in JSON format
            });
        });
      });
  }

  exports.createPreguntaWithRespuestas = function(req, res) {
    var iRespuesta = 0;
    console.log(req.body);
    waterfall([
      function(callback){
        Participante.findOne({
          usuario_id: req.body.usuario_id,
          md_perfil_clave: 'QZ',
          grupo_id: req.body.grupo_id
        },function(err, participante) {
          if(err){res.send(err);}
          callback(null, participante);
        });
      },
      function (participante, callback){
        Tematica.findOne({
          _id: req.body.tematica_id
        },function(err, tematica) {
          if(err){res.send(err);}
          callback(null, tematica, participante);
        });
      },
      function (tematica, participante, callback){
        Pregunta.create({
          titulo: req.body.titulo,
          enunciado: req.body.enunciado,
          tipo: req.body.tipo_pregunta,
          numero_respuestas: req.body.numero_respuestas,
          fecha_alta: Utils.parserDateToStr(req.body.fecha_alta),
          hora_alta: req.body.hora_alta,
          fecha_publicacion: Utils.parserDateToStr(req.body.fecha_publicacion),
          hora_publicacion: req.body.hora_publicacion,
          fecha_limite: Utils.parserDateToStr(req.body.fecha_limite),
          hora_limite: req.body.hora_limite,
          tematica_id: tematica,
          grupo_id: req.body.grupo_id,
          quizzer_id: participante
        }, function(err, pregunta) {
          if (err){
            res.send(err);
          }
          callback(null, pregunta);
        });
      },
      function (pregunta, callback){
        var respuestas = req.body.respuestas;
        var aRespuestas = JSON.parse(respuestas);
        console.log(aRespuestas);
        for(var i = 0; i < aRespuestas.length; i++){
          Respuesta.create({
            texto_respuesta: aRespuestas[i],
            pregunta_id : pregunta._id
          }, function (err, respuesta) {
            if (err){res.send(err);}
            // get and return all the respuestas after you create another
            Pregunta.findOne({            
                _id: pregunta._id
            },function (err, pregunta) {
                if (err){res.send(err);}
                pregunta.respuesta_id.push(respuesta._id);
                pregunta.save(function(err) {
                  if (err){res.send(err);}
                });
            });
          });  
        }          
        callback(null, pregunta);
      }
    ], function(err, result){
      res.json(result);
    });
  }
  
  exports.publicaPregunta = function(req, res){
    waterfall([
      function(callback){
        Pregunta.findOne({
          _id: req.params.preguntaId
        })
        .populate('tematica_id')
        .exec(function (err, pregunta){
          pregunta.fecha_publicacion = Utils.parserDateToStr(req.body.fecha_publicacion);
          pregunta.hora_publicacion = req.body.hora_publicacion;
          pregunta.save(function(err) {
            if (err){res.send(err);}
            callback(null, pregunta);
          });
        });
    },
    function (pregunta, callback){
      Grupo.findOne({
        _id: pregunta.tematica_id.grupo_id
      }, function (err, grupo){
        pregunta.nombre_grupo = grupo.nombre;
        callback(null, pregunta);
      })
    },
    function (pregunta, callback){
      Participante.find({
        grupo_id: pregunta.tematica_id.grupo_id
      })
      .populate('usuario_id')
      .exec(function (err, participantes){
        var tokens = [];
        console.log(participantes);
        for(var i in participantes){
          if(participantes[i].usuario_id.tokenGCM != null){
            tokens.push(participantes[i].usuario_id.tokenGCM);
          }
        }
        callback(null, tokens, pregunta);
      })
    },
    function(tokens, pregunta, callback){
      var sender = new gcm.Sender("AIzaSyABWJ0TfYCNm0YWvKvLcq_vdHIEhWDH-Gs");
      var message = new gcm.Message({
        collapseKey: 'demo',
        priority: 'high',
        contentAvailable: true,
        restrictedPackageName: "org.rul.quickquizz",
        data: {
            topic: 'global',
            mensaje: 'PREGUNTA_PUBLICADA',
            preguntaId: pregunta._id,
            grupo: pregunta.nombre_grupo,
            tematica: pregunta.tematica_id.nombre
        }
      });
      var registrationIds = tokens;
      console.log(message);
      sender.send(message, {'registrationIds': registrationIds}, 4, function (err, result) {
        if(err) console.error(err);
        console.log(result);
      });
      /*sender.send(message,  {'topic': ['/topics/global']}, function (err, result) {
          if(err) console.error(err);
          console.log(result);
      });*/
      callback(null, message);
    }
    ], function(err, result){
      res.json(result);
    });
  }

exports.cierraPregunta = function(req, res){
  waterfall([
    function(callback){
      Pregunta.findOne({
        _id: req.params.preguntaId
      })
        .populate('tematica_id')
        .exec(function (err, pregunta){
        pregunta.fecha_limite = Utils.parserDateToStr(req.body.fecha_limite);
        pregunta.hora_limite = req.body.hora_limite;
        pregunta.save(function(err) {
          if (err){res.send(err);}
          callback(null, pregunta);
        });
      });        
    },
    function (pregunta, callback){
      console.log(pregunta);
      Grupo.findOne({
        _id: pregunta.tematica_id.grupo_id
      }, function (err, grupo){
        pregunta.nombre_grupo = grupo.nombre;
        callback(null, pregunta);
      })
    },
    function (pregunta, callback){
      Participante.find({
        grupo_id: pregunta.tematica_id.grupo_id
      })
      .populate('usuario_id')
      .exec(function (err, participantes){
        var tokens = [];
        console.log(participantes);
        for(var i in participantes){
          if(participantes[i].usuario_id.tokenGCM != null){
            tokens.push(participantes[i].usuario_id.tokenGCM);
          }
        }
        callback(null, tokens, pregunta);
      })
    },
    function(tokens, pregunta, callback){
      var sender = new gcm.Sender("AIzaSyABWJ0TfYCNm0YWvKvLcq_vdHIEhWDH-Gs");
      var message = new gcm.Message({
        collapseKey: 'demo',
        priority: 'high',
        contentAvailable: true,
        restrictedPackageName: "org.rul.quickquizz",
        data: {
            topic: 'global',
            mensaje: 'PREGUNTA_CERRADA',
            preguntaId: pregunta._id,
            grupo: pregunta.nombre_grupo,
            tematica: pregunta.tematica_id.nombre
        }
      });
      var registrationIds = tokens;
      console.log(message);
      console.log(registrationIds);
      sender.send(message, {'registrationIds': registrationIds}, 4, function (err, result) {
        if(err) console.error(err);
        console.log(result);
      });
      /*sender.send(message,  {'topic': ['/topics/global']}, function (err, result) {
          if(err) console.error(err);
          console.log(result);
      });*/
      callback(null, message);
    }
    ], function(err, result){
      res.json(result);
    });
  }

  exports.updatePregunta = function(req, res) {
      Pregunta.findOne({
        _id: req.params.id
      },function(err, pregunta) {
        pregunta.titulo = req.body.titulo;
        pregunta.enunciado = req.body.enunciado;
        pregunta.tipo = req.body.tipo;
        pregunta.numero_respuestas = req.body.numeroRespuestas;
        pregunta.fecha_publicacion = Utils.parserDateToStr(req.body.fechaPublicacion);
        pregunta.hora_publicacion = req.body.horaPublicacion;
        pregunta.fecha_limite = Utils.parserDateToStr(req.body.fechaLimite);
        pregunta.hora_limite = req.body.horaLimite;
        pregunta.tematica_id = req.body.tematica;
        pregunta.grupo_id = req.body.grupo;

        if(err){ return handleError(err); }
        pregunta.save(function(err) {
          if (err){return handleError(err);}
          Pregunta.find({
              quizzer_id: pregunta.quizzer_id
            })
            .populate({ 
              path: 'quizzer_id',
              match: { usuario_id : req.body.usuarioId}
            })
            .populate(    
              'tematica_id'
              ).exec(function (err, preguntas) {
              // if there is an error retrieving, send the error. nothing after res.send(err) will execute
              if (err){res.send(err);}
              res.json(preguntas); // return all todos in JSON format
          });
        });
      });
  }

exports.updatePreguntaWithRespuestas = function(req, res) {
    var iRespuesta = 0;
    waterfall([
      function(callback){
        Tematica.findOne({
          _id: req.body.tematica_id
        },function(err, tematica) {
          if(err){res.send(err);}
          callback(null, tematica);
        });
      },
      function (tematica, callback){
        Pregunta.findOne({
          _id: req.params.id
        }, function (err, pregunta){
          pregunta.titulo = req.body.titulo;
          pregunta.enunciado = req.body.enunciado;
          pregunta.tipo = req.body.tipo_pregunta;
          pregunta.numero_respuestas = req.body.numero_respuestas;
          pregunta.fecha_publicacion = Utils.parserDateToStr(req.body.fecha_publicacion);
          pregunta.hora_publicacion = req.body.hora_publicacion;
          pregunta.fecha_limite = Utils.parserDateToStr(req.body.fecha_limite);
          pregunta.hora_limite = req.body.hora_limite;
          pregunta.tematica_id = tematica;
          pregunta.grupo_id = req.body.grupo_id;

          pregunta.save(function(err) {
            if (err){return handleError(err);}
            callback(null, pregunta);
          });
        });
      },
      function (pregunta, callback){
        Respuesta.remove({
          pregunta_id: pregunta._id
        }, function (err, respuestas){
          callback(null, pregunta);
        })
      },
      function (pregunta, callback){
        var respuestas = req.body.respuestas;
        var aRespuestas = JSON.parse(respuestas);
        for(var i = 0; i < aRespuestas.length; i++){
          Respuesta.create({
            texto_respuesta: aRespuestas[i],
            pregunta_id : pregunta._id
          }, function (err, respuesta) {
            if (err){res.send(err);}
            // get and return all the respuestas after you create another
            Pregunta.findOne({            
                _id: pregunta._id
            },function (err, pregunta) {
                if (err){res.send(err);}
                pregunta.respuesta_id.push(respuesta._id);
                pregunta.save(function(err) {
                  if (err){res.send(err);}
                });
            });
          });  
        }          
        callback(null, pregunta);
      }
    ], function(err, result){
      res.json(result);
    });
  }

  exports.findAll = function(req, res){
    Pregunta.find({})
        .populate('respuesta_id')
        .populate('tematica_id')
        .populate('quizzer_id')
        .populate('quizzer_id.usuario_id')
        .exec(function (err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

  exports.findAllWithoutRespuestas = function(req, res){
    Pregunta.find({})
        .populate('tematica_id')
        .populate('quizzer_id')
        .exec(function (err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

  exports.findByGrupo = function(req, res){
    Pregunta.find({})
      .populate({      
        path: 'tematica_id',
        match: { grupo_id : req.params.grupo}
      }).exec(function (err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

  exports.findByTematicas = function(req, res){
    waterfall([
      function(callback){
        Participante.find({
          usuario_id: req.params.usuarioId,
          md_perfil_clave: {$ne: 'QZ'}
        },function(err, participantes) {
          // if there is an error retrieving, send the error. nothing after res.send(err) will execute
          if (err){res.send(err);}
          var gruposIds = [];
          for (var i in participantes) {
            gruposIds.push(participantes[i].grupo_id);
          }
          callback(null, gruposIds);
        });
      },
      function(gruposIds, callback){
        Tematica.find({
            grupo_id: {$in: gruposIds}
        },function(err, tematicas){
          var tematicasIds = [];
          for(var i in tematicas){
            tematicasIds.push(tematicas[i]._id);
          }
          callback(null, tematicasIds, gruposIds);
        });
      },function(tematicasIds, gruposIds, callback){
        Pregunta.find({
          tematica_id: {$in: tematicasIds}
        }).populate('tematica_id')
          .populate('quizzer_id')
          .populate('respuesta_id')
          .exec(function (err, preguntas) {
            // if there is an error retrieving, send the error. nothing after res.send(err) will execute
            var quizzersIds = [];
            for(var i in preguntas){
              quizzersIds.push(preguntas[i].quizzer_id);
            }
            if (err){res.send(err);}
            callback(null, preguntas, quizzersIds, tematicasIds, gruposIds);
        })
      },function(preguntas, quizzersIds, tematicasIds, gruposIds, callback){
        
        Tematica.find({
          grupo_id: {$in: gruposIds}
        }).populate('grupo_id')
          .exec(function (err, tematicas){
            for(var i in preguntas){
              var pregunta = preguntas[i];
              for(var j in tematicas){
                var tematicaIdP = preguntas[i].tematica_id._id;
                var tematicaIdT = tematicas[j]._id;
                if(tematicaIdP.equals(tematicaIdT)){
                  preguntas[i].nombre_grupo = tematicas[j].grupo_id.nombre;
                  preguntas[i].grupo_id = tematicas[j].grupo_id._id;
                }
              }
            }
            callback(null, preguntas, quizzersIds, tematicasIds, gruposIds);
        });
      },function(preguntas, quizzersIds, tematicasIds, gruposIds, callback){
        var agg = [
          {$group: {
            _id: "$grupo_id",
            total: {$sum: 1}
          }}
        ];

        Participante.aggregate(agg, function(err, numeroParticipantes){
          if (err) { return res.send(err); }
          for(var i in preguntas){
            var pregunta = preguntas[i];
            for(var j in numeroParticipantes){
              var grupoIdPr = preguntas[i].tematica_id.grupo_id;
              var grupoIdPa = numeroParticipantes[j]._id;
              if(grupoIdPr.equals(grupoIdPa)){
                preguntas[i].numero_participantes = numeroParticipantes[j].total;
              }
            }
          }
          callback(null, preguntas, quizzersIds);
        });
      },function(preguntas, quizzersIds, callback){
        var agg = [
          {$group: {
            _id: "$respuesta_id",
            total: {$sum: 1}
          }}
        ];
        RespuestaParticipante.aggregate(agg, function(err, totalesRespuestas){ 
          if (err) { return res.send(err); }
          var respuestasIds = [];
          for(var i in preguntas){
            var pregunta = preguntas[i];
            var respuestas = pregunta.respuesta_id;
            var totalRespuetasPregunta = 0;
            for(var j = 0; j < respuestas.length; j++){
              var respuestaId = respuestas[j]._id;
              respuestasIds.push(respuestaId);
              for(var h in totalesRespuestas){
                var totalRespuestaId = totalesRespuestas[h]._id;
                if(respuestaId.equals(totalRespuestaId)){
                  totalRespuetasPregunta += totalesRespuestas[h].total;
                  respuestas[j].total_respuesta = totalesRespuestas[h].total;
                  break;
                }else{
                  respuestas[j].total_respuesta = 0;
                }
              }
            }
            pregunta.totales_respuestas = totalRespuetasPregunta;
          }
          callback(null, respuestasIds, preguntas, quizzersIds);
        });
      },function(respuestasIds, preguntas, quizzersIds, callback){
        RespuestaParticipante.find({
          respuesta_id: {$in: respuestasIds}
        }).populate('respuesta_id')
        .populate('participante_id')
        .exec(function (err, respuestasParticipantes){
            for(var i in preguntas){
              var pregunta = preguntas[i];
              var respuestas = pregunta.respuesta_id;
              for(var j = 0; j < respuestas.length; j++){
                var respuestaId = respuestas[j]._id;
                for(var h in respuestasParticipantes){
                  var respuestaIdRP = respuestasParticipantes[h].respuesta_id._id;
                  var usuarioIdRP = respuestasParticipantes[h].participante_id.usuario_id;
                  if(respuestaIdRP.equals(respuestaId) && usuarioIdRP.equals(req.params.usuarioId)){
                    respuestas[j].seleccionada = 1;
                    respuestas[j].respuestaParticipante_id = respuestasParticipantes[h]._id;
                    if(respuestasParticipantes[h].fecha_confirmacion != undefined && respuestasParticipantes[h].fecha_confirmacion != ""){
                      respuestas[j].confirmada = 1;
                    }else{
                      respuestas[j].confirmada = 0;
                    }
                    break;
                  }else{
                    respuestas[j].seleccionada = 0;
                    respuestas[j].confirmada = 0;
                  }
                }
              }
            }
            callback(null, preguntas, quizzersIds);
        });
      },
      function(preguntas, quizzersIds, callback){
        Participante.find({
          _id: {$in: quizzersIds}
        }).populate('usuario_id')
        .exec(function (err, participantes){
          for(var i in preguntas){
            var pregunta = preguntas[i];
            for(var j in participantes){
              var quizzerIdPr = preguntas[i].quizzer_id._id;
              var quizzerIdPa = participantes[j]._id;
              if(quizzerIdPr.equals(quizzerIdPa)){
                preguntas[i].nombre_quizzer = participantes[j].usuario_id.local.nombre;
                preguntas[i].avatar_quizzer = "generico";
                if(participantes[j].usuario_id.google != undefined && JSON.stringify(participantes[j].usuario_id.google) !== "{}"){
                  preguntas[i].nombre_quizzer = participantes[j].usuario_id.google.name;
                  preguntas[i].avatar_quizzer = participantes[j].usuario_id.google.avatar;
                }
              }
            }
          }
          callback(null, preguntas);
        });
      }
      ], function(err, result){
        //console.log(result);
        res.json(result);
      }
    )            
  }

  exports.getPreguntaByUsuario = function(req, res){
    waterfall([
      function(callback){
        Pregunta.findOne({
          _id: req.params.id
        })
        .populate('tematica_id')
        .populate('quizzer_id')
        .populate('respuesta_id')
        .exec(function (err, pregunta){
          console.log(pregunta);
          callback(null, pregunta);
        });
      },
      function (pregunta, callback){
        Tematica.findOne({
          _id: pregunta.tematica_id
        })
        .populate("grupo_id")
        .exec(function (err, tematica){
          pregunta.nombre_grupo = tematica.grupo_id.nombre;
          pregunta.grupo_id = tematica.grupo_id._id;
          callback(null, pregunta);
        });
      },
      function (pregunta, callback){
        var agg = [
          {$group: {
            _id: "$respuesta_id",
            total: {$sum: 1}
          }}
        ];
        RespuestaParticipante.aggregate(agg, function(err, totalesRespuestas){ 
          if (err) { return res.send(err); }
          var respuestasIds = [];
          var respuestas = pregunta.respuesta_id;
          
          var totalRespuetasPregunta = 0;
          for(var j = 0; j < respuestas.length; j++){
            var respuestaId = respuestas[j]._id;
            respuestasIds.push(respuestaId);
            for(var h in totalesRespuestas){
              var totalRespuestaId = totalesRespuestas[h]._id;
              if(respuestaId.equals(totalRespuestaId)){
                totalRespuetasPregunta += totalesRespuestas[h].total;
                respuestas[j].total_respuesta = totalesRespuestas[h].total;
                break;
              }else{
                respuestas[j].total_respuesta = 0;
              }
            }
          }
          pregunta.totales_respuestas = totalRespuetasPregunta;

          callback(null, respuestasIds, pregunta);
        });
      },function (respuestasIds, pregunta, callback){
        RespuestaParticipante.find({
          respuesta_id: {$in: respuestasIds}
        }).populate('respuesta_id')
        .populate('participante_id')
        .exec(function (err, respuestasParticipantes){
          
            var respuestas = pregunta.respuesta_id;
            for(var j = 0; j < respuestas.length; j++){
              var respuestaId = respuestas[j]._id;
              for(var h in respuestasParticipantes){
                var respuestaIdRP = respuestasParticipantes[h].respuesta_id._id;
                var usuarioIdRP = respuestasParticipantes[h].participante_id.usuario_id;

                console.log(respuestaId + " >> " + respuestaIdRP);
                console.log(req.params.usuarioId + " >> " + usuarioIdRP);
                if(respuestaIdRP.equals(respuestaId) && usuarioIdRP.equals(req.params.usuarioId)){
                  respuestas[j].seleccionada = 1;
                  respuestas[j].respuestaParticipante_id = respuestasParticipantes[h]._id;
                  if(respuestasParticipantes[h].fecha_confirmacion != undefined && respuestasParticipantes[h].fecha_confirmacion != ""){
                    respuestas[j].confirmada = 1;
                  }else{
                    respuestas[j].confirmada = 0;
                  }
                  break;
                }else{
                  respuestas[j].seleccionada = 0;
                  respuestas[j].confirmada = 0;
                }
              }
            }
          callback(null, pregunta);
        });
      }
      ], function(err, result){
        //console.log(result);
        res.json(result);
      }
    )            
  }

 exports.findPublicadas = function(req, res){
     waterfall([
      function(callback){
        Participante.find({
          usuario_id: req.params.usuarioId,
          md_perfil_clave: 'QZ'
        },function(err, participantes) {
          // if there is an error retrieving, send the error. nothing after res.send(err) will execute
          if (err){res.send(err);}
          var gruposIds = [];
          for (var i in participantes) {
            gruposIds.push(participantes[i].grupo_id);
          }
          callback(null, gruposIds);
        });
      },
      function(gruposIds, callback){
        Tematica.find({
            grupo_id: {$in: gruposIds}
        },function(err, tematicas){
          var tematicasIds = [];
          for(var i in tematicas){
            tematicasIds.push(tematicas[i]._id);
          }
          callback(null, tematicasIds, gruposIds);
        });
      },function(tematicasIds, gruposIds, callback){
        Pregunta.find({
          tematica_id: {$in: tematicasIds}
        }).populate('tematica_id')
          .populate('quizzer_id')
          .populate('respuesta_id')
          .exec(function (err, preguntas) {
            // if there is an error retrieving, send the error. nothing after res.send(err) will execute
            var quizzersIds = [];
            for(var i in preguntas){
              quizzersIds.push(preguntas[i].quizzer_id);
            }
            if (err){res.send(err);}
            callback(null, preguntas, quizzersIds, tematicasIds, gruposIds);
        })
      },function(preguntas, quizzersIds, tematicasIds, gruposIds, callback){
        
        Tematica.find({
          grupo_id: {$in: gruposIds}
        }).populate('grupo_id')
          .exec(function (err, tematicas){
            for(var i in preguntas){
              var pregunta = preguntas[i];
              for(var j in tematicas){
                var tematicaIdP = preguntas[i].tematica_id._id;
                var tematicaIdT = tematicas[j]._id;
                if(tematicaIdP.equals(tematicaIdT)){
                  preguntas[i].nombre_grupo = tematicas[j].grupo_id.nombre;                  
                  preguntas[i].grupo_id = tematicas[j].grupo_id._id;
                }
              }
            }
            callback(null, preguntas, quizzersIds, tematicasIds, gruposIds);
        });
      },function(preguntas, quizzersIds, tematicasIds, gruposIds, callback){
        var agg = [
          {$group: {
            _id: "$grupo_id",
            total: {$sum: 1}
          }}
        ];

        Participante.aggregate(agg, function(err, numeroParticipantes){
          if (err) { return res.send(err); }
          for(var i in preguntas){
            var pregunta = preguntas[i];
            for(var j in numeroParticipantes){
              var grupoIdPr = preguntas[i].tematica_id.grupo_id;
              var grupoIdPa = numeroParticipantes[j]._id;
              if(grupoIdPr.equals(grupoIdPa)){
                preguntas[i].numero_participantes = numeroParticipantes[j].total;
              }
            }
          }
          callback(null, preguntas, quizzersIds);
        });
      },function(preguntas, quizzersIds, callback){
        var agg = [
          {$group: {
            _id: "$respuesta_id",
            total: {$sum: 1}
          }}
        ];
        RespuestaParticipante.aggregate(agg, function(err, totalesRespuestas){ 
          if (err) { return res.send(err); }
          var respuestasIds = [];
          for(var i in preguntas){
            var pregunta = preguntas[i];
            var respuestas = pregunta.respuesta_id;
            var totalRespuetasPregunta = 0;
            for(var j = 0; j < respuestas.length; j++){
              var respuestaId = respuestas[j]._id;
              respuestasIds.push(respuestaId);
              for(var h in totalesRespuestas){
                var totalRespuestaId = totalesRespuestas[h]._id;
                if(respuestaId.equals(totalRespuestaId)){
                  totalRespuetasPregunta += totalesRespuestas[h].total;
                  respuestas[j].total_respuesta = totalesRespuestas[h].total;
                  break;
                }else{
                  respuestas[j].total_respuesta = 0;
                }
              }
            }
            pregunta.totales_respuestas = totalRespuetasPregunta;
          }
          callback(null, respuestasIds, preguntas, quizzersIds);
        });
      },function(respuestasIds, preguntas, quizzersIds, callback){
        RespuestaParticipante.find({
          respuesta_id: {$in: respuestasIds}
        }).populate('respuesta_id')
        .populate('participante_id')
        .exec(function (err, respuestasParticipantes){
            for(var i in preguntas){
              var pregunta = preguntas[i];
              var respuestas = pregunta.respuesta_id;
              for(var j = 0; j < respuestas.length; j++){
                var respuestaId = respuestas[j]._id;
                for(var h in respuestasParticipantes){
                  var respuestaIdRP = respuestasParticipantes[h].respuesta_id._id;
                  var usuarioIdRP = respuestasParticipantes[h].participante_id.usuario_id;
                  if(respuestaIdRP.equals(respuestaId) && usuarioIdRP.equals(req.params.usuarioId)){
                    respuestas[j].seleccionada = 1;
                    respuestas[j].respuestaParticipante_id = respuestasParticipantes[h]._id;
                    if(respuestasParticipantes[h].fecha_confirmacion != undefined && respuestasParticipantes[h].fecha_confirmacion != ""){
                      respuestas[j].confirmada = 1;
                    }else{
                      respuestas[j].confirmada = 0;
                    }
                    break;
                  }else{
                    respuestas[j].seleccionada = 0;
                    respuestas[j].confirmada = 0;
                  }
                }
              }
            }
            callback(null, preguntas, quizzersIds);
        });
      },
      function(preguntas, quizzersIds, callback){
        Participante.find({
          _id: {$in: quizzersIds}
        }).populate('usuario_id')
        .exec(function (err, participantes){
          for(var i in preguntas){
            var pregunta = preguntas[i];
            for(var j in participantes){
              var quizzerIdPr = preguntas[i].quizzer_id._id;
              var quizzerIdPa = participantes[j]._id;
              if(quizzerIdPr.equals(quizzerIdPa)){
                preguntas[i].nombre_quizzer = participantes[j].usuario_id.local.nombre;
                preguntas[i].avatar_quizzer = "generico";
                if(participantes[j].usuario_id.google != undefined && JSON.stringify(participantes[j].usuario_id.google) !== "{}"){
                  preguntas[i].nombre_quizzer = participantes[j].usuario_id.google.name;
                  preguntas[i].avatar_quizzer = participantes[j].usuario_id.google.avatar;
                }
              }
            }
          }
          callback(null, preguntas);
        });
      }
      ], function(err, result){
        //console.log(result);
        res.json(result);
      }
    )            
  }

  exports.findPublicadasByFechas = function(req, res){
    Pregunta.find({
        fecha_publicacion:  { $gt: req.params.fechaInicio, $lt: req.params.fechaFin },

      },
      null,
      {
        skip:0, // Starting Row
        limit:10, // Ending Row
        sort:{
            fecha_publicacion: -1 //Sort by Date Added DESC
        }
      }).populate({      
        path: 'quizzer_id',
        match: { email : req.params.email}
      }).exec(function (err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

  exports.findVigentes = function(req, res){

    Pregunta.find({
        fecha_limite:  { $gt: req.params.dataActual}
      },function(err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

    exports.findPublicadasByGrupo = function(req, res){

    Pregunta.find({
        fecha_publicacion:  { $gt: req.params.fechaInicio, $lt: req.params.fechaFin },
        grupo_id: req.params.grupo
      },function(err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

  exports.findPublicadasByTematica = function(req, res){

    Pregunta.find({
        fecha_publicacion:  { $gt: req.params.fechaInicio, $lt: req.params.fechaFin },
        tematica_id: req.params.tematica
      },function(err, preguntas) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(preguntas); // return all todos in JSON format
    });
  }

  exports.getPregunta = function(req, res){
    Pregunta.findOne({
        _id: req.params.id
      })
      .populate(    
        'tematica_id'
        ).exec(function (err, pregunta) {
        // if there is an error retrieving, send the error. nothing after res.send(err) will execute
        if (err){
            res.send(err);
        }
        res.json(pregunta); // return all todos in JSON format
    });
  }

  // first locates a thread by title, then locates the replies by thread ID.
  exports.deletePregunta = function(req, res) {
      Pregunta.remove({
          _id : req.params.id
      }, function(err, pregunta) {
          if (err) res.send(err);
          res.json({ 
            message: "Pregunta eliminada."
          });
      });
  }
