var RespuestaParticipante = require('../models/respuesta_participante.js');
var Respuesta = require('../models/respuesta.js');
var Participante = require('../models/participante.js');
var Utils = require('../utils/utils.js');
var waterfall = require('async-waterfall');

exports.createRespuestaParticipante = function(req, res) {
  console.log(req.body);
  RespuestaParticipante.create({
    puntuacion: req.body.puntuacion,
    fecha_respuesta: Utils.parserDateToStr(req.body.fechaRespuesta),
    hora_respuesta: req.body.horaRespuesta,
    participante_id : req.body.participanteId,
    respuesta_id: req.body.respuestaId
  }, function(err, respuestaParticipante) {
      if (err){
        res.send(err);
      }
      // get and return all the respuestas after you create another
      RespuestaParticipante.find(function(err, respuestasParticipante) {
          if (err){
            res.send(err);
          }
          res.json(respuestasParticipante);
      });
  });
}

exports.checkRespuestaCheckbox = function(req, res){
  waterfall([
    function (callback){
      Respuesta.findOne({
        _id: req.body.respuesta
      }, function (err, respuesta){
        if(err) return handleError(err);
        callback(null, respuesta);
      });
    },
    function (respuesta, callback){
      Participante.findOne({
        usuario_id: req.body.usuario,
        grupo_id: req.body.grupo_id
      }, function (err, participante){
        if(err) return handleError(err);
        callback(null, participante, respuesta);
      });
    },
    function (participante, respuesta, callback){
      var valor = 0;
          if(respuesta.es_correcta == 'S'){
            valor += respuesta.valor_correcta;
          }else if(respuesta.es_correcta == 'N'){
            valor += respuesta.valor_incorrecta;
          }
          RespuestaParticipante.create({
            puntuacion: valor,
            fecha_respuesta: Utils.parserDateToStr(req.body.fechaRespuesta),
            hora_respuesta: req.body.horaRespuesta,
            participante_id: participante._id,
            respuesta_id: req.body.respuesta
          }, function (err, respuestaParticipante){
            if(err) return handleError(err);
              // get and return all the respuestas after you create another
              RespuestaParticipante.find(function(err, respuestasParticipante) {
                  if (err){
                    res.send(err);
                  }
                  callback(null, respuestasParticipante);
              });
          })
    }
    ],function (err, result){
        res.json(result);
    }
  );
}
exports.uncheckRespuestaCheckbox = function(req, res){
  waterfall([
    function (callback){
      Participante.findOne({
        usuario_id: req.body.usuario,
        grupo_id: req.body.grupo_id
      }, function (err, participante){
        if(err) return handleError(err);
        callback(null, participante);
      })
    },function (participante, callback) {
      RespuestaParticipante.remove({
        participante_id: participante._id,
        respuesta_id: req.body.respuesta
      }, function (err, respuestaParticipante) {
        if (err)
            res.send(err);

        // get and return all the calificaciones after you create another
        Respuesta.find(function (err, respuestasParticipante) {
            if (err){res.send(err);}
            callback(null, respuestasParticipante);
        });
      });
    }  
    ],function (err, result){
        res.json(result);
    }
  );
}
exports.checkRespuestaRadio = function(req, res){
  waterfall([
    function (callback){
      Respuesta.findOne({
        _id: req.body.respuesta
      }, function (err, respuesta){
        if(err) return handleError(err);
        callback(null, respuesta);
      });
    },
    function (respuesta, callback){                      //Recuperamos el participante
      Participante.findOne({
        usuario_id: req.body.usuario,
        grupo_id: req.body.grupo_id
      }, function (err, participante){
        if(err) return handleError(err);
        callback(null, participante, respuesta);
      });
    },
    function (participante, respuesta, callback){        //Recuperamos las respuestas de la pregunta
      Respuesta.find({
        pregunta_id: req.body.pregunta
      }, function (err, respuestas){
        if (err){res.send(err);}
        var respuestasIds = [];
        for (var i in respuestas) {
          respuestasIds.push(respuestas[i]._id);
        }
        callback(null, respuestasIds, participante, respuesta);
      });
    },
    function (respuestasIds, participante, respuesta, callback){ //Eliminamos las respuestas_participantes que ya teniamos seleccionadas
      RespuestaParticipante.remove({
        participante_id: participante._id,
        respuesta_id: {$in: respuestasIds}
      }, function (err, respuestasEliminadas) {
        if (err){ res.send(err);}
        callback(null, participante, respuesta);
      });
    },
    function (participante, respuesta, callback){
      var valor = 0;
          if(respuesta.es_correcta == 'S'){
            valor += respuesta.valor_correcta;
          }else if(respuesta.es_correcta == 'N'){
            valor += respuesta.valor_incorrecta;
          }
          RespuestaParticipante.create({
            puntuacion: valor,
            fecha_respuesta: Utils.parserDateToStr(req.body.fechaRespuesta),
            hora_respuesta: req.body.horaRespuesta,
            participante_id: participante._id,
            respuesta_id: req.body.respuesta
          }, function (err, respuestaParticipante){
            if(err) return handleError(err);
              // get and return all the respuestas after you create another
              RespuestaParticipante.find(function(err, respuestasParticipante) {
                  if (err){
                    res.send(err);
                  }
                  callback(null, respuestasParticipante);
              });
          })
    }
    ],function(err, result){
        res.json(result);
    }
  );
}
exports.confirmRespuestaParticipante = function(req, res){
  Participante.findOne({
    usuario_id: req.body.usuario_id,
    grupo_id: req.body.grupo_id
  }, function (err, participante){
    if(err){ return res.send(err);}
    console.log(participante);
    RespuestaParticipante.findOne({
      participante_id: participante._id,
      respuesta_id: req.body.respuesta_id
    }, function (err, respuestaParticipante){
      if(err){ return res.send(err);}
      
      respuestaParticipante.fecha_confirmacion = Utils.parserDateToStr(req.body.fechaConfirmacion);
      respuestaParticipante.hora_confirmacion = req.body.horaConfirmacion;
      console.log(respuestaParticipante);
      respuestaParticipante.save(function(err) {
        if (err){
          return handleError(err);
        }
        res.json({ 
          message: "Respuesta confirmada."
        });
      }); 
    });
  });
}

exports.findAll = function(req, res){
  RespuestaParticipante.find()
    .populate("participante_id")
    .populate("respuesta_id")
    .exec(function (err, respuestasParticipante) {
    // if there is an error retrieving, send the error. nothing after res.send(err) will execute
    if (err){
        res.send(err);
    }
    res.json(respuestasParticipante); // return all todos in JSON format
  });
}

exports.findRespuestasByParticipante = function(req, res){
  RespuestaParticipante.find({
    pregunta_id: req.params.preguntaParticipante
  },function(err, respuestasParticipante){
    if(err){
      res.send(err);
    }
    res.json(respuestasParticipante);
  });
}

exports.getRespuestaParticipante = function(req, res){
  RespuestaParticipante.findOne({
    _id: req.params.id
  }, function(err, respuestaParticipante){
    if(err){
      res.send(err);
    }
    res.json(respuestaParticipante);
  });
}

// first locates a thread by title, then locates the replies by thread ID.
exports.deleteRespuestaParticipante = function(req, res) {
  RespuestaParticipante.remove({
      _id : req.params.id
  }, function(err, respuestaParticipante) {
      if (err)
          res.send(err);

      // get and return all the calificaciones after you create another
      RespuestaParticipante.find(function(err, respuestasParticipante) {
          if (err)
              res.send(err)
          res.json(respuestasParticipante);
      });
  });
}

exports.findEstadisticasResultados = function (req, res){
  waterfall([
    function (callback){
      Respuesta.find({
        pregunta_id: req.params.preguntaId
      }, function (err, respuestas){
        callback(null, respuestas);
      })
    },
    function (respuestas, callback){
      var agg = [
        {$group: {
          _id: "$respuesta_id",
          total: {$sum: 1}
        }}
      ];
      RespuestaParticipante.aggregate(agg, function(err, totalesRespuestas){ 
        if (err) { return res.send(err); }
        var totalRespuestas = 0;
        var jPorcentajes = {porcentajes: []};
        for(var i = 0; i < totalesRespuestas.length; i++){
            totalRespuestas += totalesRespuestas[i].total;
        }
        console.log(respuestas);
        for(var j in respuestas){
          var porcentajeRegistrado = false;
          for(var i = 0; !porcentajeRegistrado && i < totalesRespuestas.length; i++){
            if(respuestas[j]._id.equals(totalesRespuestas[i]._id)){
              jPorcentajes.porcentajes.push(totalesRespuestas[i].total * 100/totalRespuestas);
              porcentajeRegistrado = true;
            }
          }
          if(!porcentajeRegistrado){
            jPorcentajes.porcentajes.push(0.00);
          }
        }  
        callback(null, jPorcentajes);
      });
    }
    ], function(err, result){
      res.json(result);
    }
  );
}