var preguntaController = angular.module('preguntaController', []);

preguntaController.controller('fichaPreguntaCtrl', ['$scope', '$log', '$window', 'grupoSrv', 'tematicaSrv', 'preguntaSrv',
  'respuestaSrv', function($scope, $log, $window, grupoSrv, tematicaSrv, preguntaSrv, respuestaSrv) {
    function limpiarPregunta($scope){
      $scope.titulo = "";
      $scope.enunciado = "";
      $scope.fechapublicacion = "";
      $scope.fechalimite = "";
      $scope.horapublicacion = "";
      $scope.horalimite = "";
      $scope.tipo = "";
      $scope._id = "";
      $scope.grupo = "";
      $scope.tematica = "";
      $scope.respuesta = {};
      $scope.respuestas = [];
    }

    $scope.usuarioid = $window.usuarioid;  
    $scope.tipospregunta = ['MONORESPUESTA', 'MULTIRESPUESTA', 'RANGO'];
    $scope.correcincorrecs = [{label: 'CORRECTA', value: 'S'}, {label: 'INCORRECTA', value: 'N'}];
    grupoSrv.getGruposQuizzer($scope.usuarioid).then(function(dataResponse) {
      $scope.grupos = dataResponse.data;
    });
    preguntaSrv.getMisPreguntas($scope.usuarioid).then(function(dataResponse) {
        $scope.preguntas = dataResponse.data;
    });
    $scope.tematicas = [];
    // $scope que acciona el ng-change
    $scope.cargarTematicas = function() { 
      // $scope.grupo NOS TRAE EL VALOR DEL SELECT DE Grupos
      tematicaSrv.getTematicasByGrupo($scope.grupo).then(function(dataResponse) {
        $scope.tematicas = dataResponse.data;
      });
    };
    
    $scope.guardarPregunta = function() {
        var pregunta = {};          
        pregunta._id = $scope._id;
        pregunta.titulo = $scope.titulo;
        pregunta.enunciado = $scope.enunciado;
        pregunta.fechaAlta = formatDateToString(new Date());
        pregunta.fechaPublicacion = formatDateToString($scope.fechapublicacion);
        pregunta.fechaLimite = formatDateToString($scope.fechalimite);
        pregunta.horaPublicacion = formatTimeToMillis($scope.horapublicacion);
        pregunta.horaLimite = formatTimeToMillis($scope.horalimite)
        pregunta.grupo = $scope.grupo;
        pregunta.tematica = $scope.tematica;
        pregunta.usuarioId = $scope.usuarioid;
        pregunta.tipo = $scope.tipo;
        if(pregunta._id == undefined || pregunta._id == null){
          preguntaSrv.createPregunta(pregunta).then(function(dataResponse) {
            $scope.preguntas = dataResponse.data.preguntas;
            $scope._id = dataResponse.data.preguntaId;
            $scope.respuesta = {};
            $scope.respuestas = [];
          });
        }else{
          preguntaSrv.updatePregunta(pregunta).then(function(dataResponse) {
            $scope.preguntas = dataResponse.data;
          });
        }
    };

    $scope.cargarPregunta = function(pregunta){
        $scope.titulo = pregunta.titulo;
        $scope.enunciado = pregunta.enunciado;
        $scope.fechapublicacion = parserStrToDate(pregunta.fecha_publicacion);
        $scope.fechalimite = parserStrToDate(pregunta.fecha_limite);
        $scope.horapublicacion = parseMilisToTime(pregunta.hora_publicacion);
        $scope.horalimite = parseMilisToTime(pregunta.hora_limite);
        $scope.tipo = pregunta.tipo;
        $scope._id = pregunta._id;
        grupoSrv.getGrupoByTematica(pregunta.tematica_id._id).then(function(dataResponse){
          $scope.grupo = dataResponse.data._id;
          tematicaSrv.getTematicasByGrupo($scope.grupo).then(function(dataResponse) {
            $scope.tematicas = dataResponse.data;
            $scope.tematica = pregunta.tematica_id._id;
          });          
        });

        respuestaSrv.getRespuestasByPregunta(pregunta._id).then(function(dataResponse){
          $scope.respuestas = dataResponse.data;
        });
        $scope.respuesta = {};
    };

    $scope.limpiarPregunta = function(){
      limpiarPregunta($scope);
    }

    $scope.deletePregunta = function(pregunta){
      if(pregunta._id == $scope._id){
        limpiarPregunta($scope);
      }
      preguntaSrv.deletePregunta(pregunta).then(function(dataResponse){
        $scope.preguntas = dataResponse.data;
      });
    }
    $scope.cargarRespuesta = function(respuesta){
      $scope.respuesta = {};
      $scope.respuesta._id = respuesta._id;
      $scope.respuesta.texto = respuesta.texto_respuesta;
      $scope.respuesta.valor_correcta = respuesta.valor_correcta;
      $scope.respuesta.valor_incorrecta = respuesta.valor_incorrecta;
      $scope.respuesta.es_correcta = respuesta.es_correcta;
    };

    $scope.guardarRespuesta = function(){
      var respuesta = {};
      console.log($scope);
      respuesta._id = $scope.respuesta._id;
      respuesta.texto_respuesta = $scope.respuesta.texto;
      respuesta.valor_incorrecta = $scope.respuesta.valor_incorrecta;
      respuesta.valor_correcta = $scope.respuesta.valor_correcta;
      respuesta.es_correcta = $scope.respuesta.es_correcta;
      respuesta.pregunta_id = $scope._id;
      if(respuesta._id == undefined || respuesta._id == null){
        respuestaSrv.createRespuesta(respuesta).then(function(dataResponse){
          $scope.respuestas = dataResponse.data;
        });
      }else{        
        respuestaSrv.updateRespuesta(respuesta).then(function(dataResponse){
          $scope.respuestas = dataResponse.data;
        });
      }
      $scope.respuesta = {};
    };

    $scope.limpiarRespuesta = function(){
      $scope.respuesta = {};
    }

    $scope.deleteRespuesta = function(respuesta){
      if(respuesta._id == $scope.respuesta._id){
        $scope.respuesta = {};
      }
      respuestaSrv.deleteRespuesta(respuesta).then(function(dataResponse){
        $scope.respuestas = dataResponse.data;
      });
    }
}]);

preguntaController.controller("dialogCtrl", function($scope, $log, $window, $mdDialog, pregunta, grupoSrv, usuarioSrv, respuestaSrv) {
  $scope.pregunta = pregunta;
  $scope.tematica = pregunta.tematica_id.nombre;
  grupoSrv.getGrupo(pregunta.tematica_id.grupo_id).then(function(dataResponse) {
    $scope.grupo = dataResponse.data;
  });
  usuarioSrv.getUsuario(pregunta.quizzer_id.usuario_id).then(function(dataResponse){
    $scope.usuario = dataResponse.data;
  });

  //$scope.respuestasJSON = JSON.parse($scope.respuestas);
  $scope.dataIniciada = formatStringFecha(pregunta.fecha_publicacion);
  $scope.dataFinalizada = formatStringFecha(pregunta.fecha_limite);

  $scope.respuestaSelected = null;
  respuestaSrv.getRespuestasByPregunta(pregunta._id).then(function(dataResponse){
    $scope.respuestas = dataResponse.data;
  });  

  $scope.hide = function() {
     $mdDialog.hide();
  };
  $scope.cancel = function() {
    $mdDialog.cancel();
  };
  $scope.confirmarRespuesta = function(tipo) {
    console.log(tipo);
    console.log($scope);
    if(tipo == "MONORESPUESTA"){
    }
    $mdDialog.cancel();
  };
  $scope.mostrarResultados = function(tipo) {
    $mdDialog.hide();
  };

});

preguntaController.controller("preguntaCtrl", ['$scope', '$log', '$mdDialog', '$window', 'preguntaSrv', 'respuestaSrv', 
  function($scope, $log, $mdDialog, $window, preguntaSrv, respuestaSrv) {
    $scope.usuarioid = $window.usuarioid;   

    //Obtenemos las preguntas que yo he publicado
    preguntaSrv.getMisPreguntas($scope.usuarioid).then(function(dataResponse) {
        $scope.misPreguntas = dataResponse.data;
    });

    preguntaSrv.getPreguntasMisGrupos($scope.usuarioid).then(function(dataResponse) {
        $scope.preguntas = dataResponse.data;
    });

    $scope.showPregunta = function(ev, pregunta) {
      console.log("Show modal");
      $mdDialog.show({
        controller: 'dialogCtrl',
        templateUrl: '/template/modalPregunta.tmpl.html',
        parent: angular.element(document.body),
        targetEvent: ev,
        scope: $scope,
        locals: {
          pregunta: pregunta
        }
      })
      .then(function(answer) {
        $scope.alert = 'You said the information was "' + answer + '".';
      }, function() {
        $scope.alert = 'You cancelled the dialog.';
      });
    };
}]);

preguntaController.directive("qqPregunta",[function() {
   
  function calcularTiempoRestante(pregunta){
    var duracion =  "Sin límite";
    if(pregunta.fecha_limite == ""){
      duracion = "Sin límite"
    }else{
      var fl = pregunta.fecha_limite,
        dl = fl.substr(6, 2),
        ml = fl.substr(4, 2),
        al = fl.substr(0, 4);
      
      var dateP = new Date();
      var dateL = new Date(al, ml, dl);
      var tp = dateP.getTime();
      var tl = dateL.getTime();

      var dias = parseInt((tl-tp)/(24*3600*1000));
      var horas = parseInt((pregunta.hora_limite / (3600*1000)) - dateP.getHours());
      var restoHoras = parseInt(pregunta.hora_limite % (3600*1000));
      var minutosLimite = parseInt(restoHoras / (60*1000));
      if(minutosLimite == 0){
        minutosLimite = 60;
        horas--;
      }
      var minutos = minutosLimite - dateP.getMinutes();
      duracion = "Hasta: " + dias + "d " + horas + "h " + minutos + "m";
    }
    return ((pregunta.hora_limite + tl) - tp) / 1000;
  }

  var directiveDefinitionObject ={
    restrict:"E",
    replace : true,
    transclude: true,
    scope: {
      mipregunta: "=mipregunta",
      respuestas: "@",
      tematica: "@",
      grupoid: "@",
      tipo:"@"
    },
    templateUrl: '/template/cardPregunta.tmpl.html',
    controller: function($scope, $element, $log, grupoSrv){
      var mipregunta = $scope.mipregunta;
      grupoSrv.getGrupo($scope.grupoid).then(function(dataResponse) {
        $scope.grupo = dataResponse.data;
      });
      //$scope.respuestasJSON = JSON.parse($scope.respuestas);
      $scope.duracion = calcularTiempoRestante(mipregunta);
    }
  }
   
  return directiveDefinitionObject;
}]);

preguntaController.directive("qqPreguntaPublicada",[function() {
   
  var directiveDefinitionObject ={
    restrict:"E",
    replace : true,
    transclude: true,
    scope: {
      pregunta: "=pregunta",
      tematica: "@",
      grupoid: "@",
      usuarioid:"@"
    },
    templateUrl: '/template/rowPregunta.tmpl.html',
    controller: function($scope, $element, $log, grupoSrv, usuarioSrv){
      var pregunta = $scope.pregunta;
      if(pregunta != undefined){
        grupoSrv.getGrupo($scope.grupoid).then(function(dataResponse) {
          $scope.grupo = dataResponse.data;
        });
        usuarioSrv.getUsuario($scope.usuarioid).then(function(dataResponse){
          $scope.usuario = dataResponse.data;
        });
        //$scope.respuestasJSON = JSON.parse($scope.respuestas);
        $scope.dataIniciada = formatStringFecha(pregunta.fecha_publicacion);
        $scope.dataFinalizada = formatStringFecha(pregunta.fecha_limite);
      }
    }
  }
   
  return directiveDefinitionObject;
}]);

preguntaController.directive("qqItemFichaPregunta",[function() {
   
  var directiveDefinitionObject ={
    restrict:"E",
    replace : true,
    transclude: true,
    scope: {
      pregunta: "=pregunta",
      tematica: "@",
      grupoid: "@",
      usuarioid:"@"
    },
    templateUrl: '/template/itemFichaPregunta.tmpl.html',
    controller: function($scope, $element, $log, grupoSrv, usuarioSrv){
      var pregunta = $scope.pregunta;
      if(pregunta != undefined){
        grupoSrv.getGrupo($scope.grupoid).then(function(dataResponse) {
          $scope.grupo = dataResponse.data;
        });
        usuarioSrv.getUsuario($scope.usuarioid).then(function(dataResponse){
          $scope.usuario = dataResponse.data;
        });
        //$scope.respuestasJSON = JSON.parse($scope.respuestas);
        $scope.dataIniciada = formatStringFecha(pregunta.fecha_publicacion);
        $scope.dataFinalizada = formatStringFecha(pregunta.fecha_limite);
      }
    }
  }
   
  return directiveDefinitionObject;
}]);

preguntaController.directive("qqRespuestas",[function() {
   
  var directiveDefinitionObject = {
    require: '^qqPregunta',
    restrict:"E",
    replace : true,
    scope: {
      tipo: "@",
      respuestas: "@",
      preguntaid: "@"
    },
    template: 
              '<div flex="100" ng-switch="tipo">' +
                '<div ng-switch-when="MULTIRESPUESTA">' +
                  '<div ng-repeat="respuesta in respuestas track by $index">' +
                    '<md-checkbox>' +
                        '{{ respuesta.texto_respuesta }} <span ng-if="exists(respuesta, selected)">selected</span>' +
                    '</md-checkbox>' +
                  '</div>' +
                '</div>' +
                '<div ng-switch-when="MONORESPUESTA">' +
                  '<md-radio-group ng-model="respuestaSelected">' +
                    '<md-radio-button ng-repeat="respuesta in respuestas  track by $index" value="{{ respuesta._id }}" class="md-primary">{{ respuesta.texto_respuesta }}</md-radio-button>' +
                  '</md-radio-group>' +
                  //'<code>respuestaSelected={{respuestaSelected}}</code>' +
                '</div>' +
                '<div ng-switch-when="RANGO">Home Span</div>' +
                '<div ng-switch-default>default</div>' +
                //'<code>tipo={{tipo}}</code>' +
              '</div>',
    controller: function($scope, $element, $log, respuestaSrv){
      //$log.debug($scope.preguntaId);
      //$scope.respuestas = respuestaSrv.getRespuestasByPregunta($scope.preguntaId);
      $log.debug($scope.respuestas);
    }
  }
   
  return directiveDefinitionObject;
}]);

function formatStringFecha(sFecha){
    var d = sFecha.substr(6, 2),
      m = sFecha.substr(4, 2),
      a = sFecha.substr(0, 4);
    
    var dFecha = new Date(a, m, d);
    return dFecha;
}

function formatDateToString(dFecha){
  if(dFecha != undefined && dFecha != null){
    var dd = dFecha.getDate(),
      mm = dFecha.getMonth() + 1,
      yyyy = dFecha.getFullYear();

    return ((dd>9)?dd:"0"+dd) + '/' + ((mm>9)?mm:"0"+mm) + '/' + yyyy;
  }else{
    return "";
  }
}

function formatTimeToMillis(dHora){
  return moment.utc(dHora).unix();
}

function parserStrToDate(str) {
    if(!/^(\d){8}$/.test(str)) return "invalid date";
    var y = str.substr(0,4),
        m = str.substr(4,2),
        d = str.substr(6,2);
    return new Date(y,m,d);
}

function parseMilisToTime(milis) {
    return moment.unix(milis).toDate();
}