var api = require('../controllers/api.js');
var usuario = require('../controllers/usuario');
var calificacion = require('../controllers/calificacion');
var grupo = require('../controllers/grupo');
var perfil = require('../controllers/md_perfil');
var participante = require('../controllers/participante');
var tematica = require('../controllers/tematica');
var pregunta = require('../controllers/pregunta');
var respuesta = require('../controllers/respuesta');
var respuestaParticipante = require('../controllers/respuesta_participante');

function setup(app, passport) {

	//routes Usuario
	app.get('/api/usuarios', usuario.findAll);
	//app.get('/api/usuario/:id', usuario.findById);
  	app.post('/api/usuario/local', usuario.createUsuario);
	app.get('/api/usuario/email/:email', usuario.findByEmail);
	app.get('/api/usuario/google/:email', usuario.findGoogleByEmail);
	app.get('/api/usuario/:id', usuario.getUsuario);
	app.put('/api/usuario/gcm/registrar/:id', usuario.registrarGCM);
	app.put('/api/usuario/gcm/desregistrar/:id', usuario.desregistrarGCM);
	app.post('/sendGCM', usuario.sendGCM);
	app.post('/api/usuario/google/signup', usuario.googleSignUp);
	app.post('/api/usuario/google', usuario.createUsuarioGoogle);
	app.put('/api/usuario/:email', usuario.updateUsuario);
	app.delete('/api/usuario/:email', usuario.deleteUsuario);

	//routes Calificacion
	app.get('/api/calificaciones', calificacion.findAll);
	app.get('/api/calificacion/:participante_id', calificacion.findByParticipante);
	app.get('/api/calificacion/:tematica_id', calificacion.findByTematica);
	app.get('/api/calificacion/:participante_id/tematica_id', calificacion.getCalificacion);
	app.post('/api/calificacion', calificacion.createCalificacion);
	app.put('/api/calificacion/:participante_id/tematica_id', calificacion.updateCalificacion);
	app.delete('/api/calificacion/:participante_id/tematica_id', calificacion.deleteCalificacion);

	//routes Grupo
	app.get('/api/grupos', grupo.findAll);
	app.get('/api/grupos/usuario/:usuarioId', grupo.findByQuizzer);
	app.get('/api/grupo/:id', grupo.getGrupo);
	app.get('/api/grupo/tematica/:tematicaId', grupo.findByTematica);
	app.get('/api/grupo/nombre/:nombre', grupo.findByNombre);
	app.get('/api/grupos/publicos', grupo.findAllPublicos);
	app.post('/api/grupo', grupo.createGrupo);
	app.put('/api/grupo/:id', grupo.updateGrupo);
	app.delete('/api/grupo/id', grupo.deleteGrupo);

	//routes MD_Perfil
	app.get('/api/perfiles', perfil.findAll);
	app.post('/api/perfil', perfil.createPerfil);
	app.delete('/api/perfil/:clave', perfil.deletePerfil);

	//routes Participante
	app.get('/api/participantes', participante.findAll);
	app.get('/api/participantes/grupo/:grupo', participante.findParticipantesByGrupo);
	app.get('/api/participante/email/:email', participante.findParticipantesByEmail);
	app.get('/api/participante/:id', participante.getParticipante);
	app.post('/api/participante', participante.createParticipante);
	app.delete('/api/participante/:clave', participante.deleteParticipante);
	app.delete('/api/participante/grupo/:grupo_id/:usuario_id', participante.dejarGrupo);

	//routes Tematica
	app.get('/api/tematicas', tematica.findAll);
	app.get('/api/tematicas/grupo/:grupoId', tematica.findByGrupo);
	app.get('/api/tematicas/quizzer/:usuarioId', tematica.findByQuizzer);
	app.post('/api/tematica', tematica.createTematica);
	app.put('/api/tematica/:id', tematica.updateTematica);
	app.delete('/api/tematica/:id', tematica.deleteTematica);

	//routes Preguntas
	app.get('/api/preguntas', pregunta.findAll);
	app.get('/api/preguntas/sinrespuestas', pregunta.findAllWithoutRespuestas);
	app.get('/api/preguntas/grupo/:grupo', pregunta.findByGrupo);
	app.get('/api/preguntas/tematicas/:usuarioId', pregunta.findByTematicas);
	app.get('/api/preguntas/vigentes/:dataActual', pregunta.findVigentes);
	app.get('/api/preguntas/usuario/:usuarioId', pregunta.findPublicadas);
	app.get('/api/preguntas/:fechaInicio/:fechaFin', pregunta.findPublicadasByFechas);
	app.get('/api/preguntas/grupo/:grupo/:fechaInici/:fechaFin', pregunta.findPublicadasByGrupo);
	app.get('/api/preguntas/tematica/:tematica/:fechaInici/:fechaFin', pregunta.findPublicadasByTematica);
	app.get('/api/pregunta/:id', pregunta.getPregunta);
	app.get('/api/pregunta/:id/:usuarioId', pregunta.getPreguntaByUsuario);
	app.post('/api/pregunta', pregunta.createPregunta);
	app.post('/api/pregunta_w_respuestas', pregunta.createPreguntaWithRespuestas);
	app.put('/api/pregunta/publicar/:preguntaId', pregunta.publicaPregunta);
	app.put('/api/pregunta/cerrar/:preguntaId', pregunta.cierraPregunta);
	app.put('/api/pregunta/:id', pregunta.updatePreguntaWithRespuestas);
	app.delete('/api/pregunta/:id', pregunta.deletePregunta);

	//routes Respuesta
	app.get('/api/respuestas', respuesta.findAll);
	app.get('/api/respuestas/:id', respuesta.getRespuesta);
	app.get('/api/respuestas/pregunta/:pregunta', respuesta.findByPregunta);
	app.post('/api/respuesta', respuesta.createRespuesta);
	app.put('/api/respuesta/:id', respuesta.updateRespuesta);
	app.delete('/api/respuesta/:id/:preguntaid', respuesta.deleteRespuesta);

	//routes Respuesta Participante
	app.get('/api/respuestas_participantes', respuestaParticipante.findAll);
	app.get('/api/respuesta_participante/:id', respuestaParticipante.getRespuestaParticipante);
	app.get('/api/respuestas_participantes/:participante', respuestaParticipante.findRespuestasByParticipante);
	app.post('/api/respuesta_participante', respuestaParticipante.createRespuestaParticipante);
	app.post('/api/respuesta_participante/checkbox/check', respuestaParticipante.checkRespuestaCheckbox);
	app.post('/api/respuesta_participante/checkbox/uncheck', respuestaParticipante.uncheckRespuestaCheckbox);
	app.post('/api/respuesta_participante/radio/check', respuestaParticipante.checkRespuestaRadio);
	app.put('/api/respuesta_participante/confirm', respuestaParticipante.confirmRespuestaParticipante);
	app.delete('/api/respuesta_participante/:id', respuestaParticipante.deleteRespuestaParticipante);
	app.get('/api/respuesta_participante/resultados/:preguntaId', respuestaParticipante.findEstadisticasResultados);

	// normal routes ===============================================================
    // show the home page (will also have our login links)
    // normal routes ===============================================================

	// show the home page (will also have our login links)
	app.get('/', function(req, res) {
		res.render('pages/index');
	});

	app.get('/dashboard', isLoggedIn, function(req, res) {
		res.render('pages/dashboard', {
			user : req.user,
			toolbar: {
				titulo : "Dashboard"
			}
		});
	});

	app.get('/pregunta', isLoggedIn, function(req, res) {
		res.render('pages/pregunta', {
			user : req.user,
			toolbar: {
				titulo : "Preguntas"
			}
		});
	});

	app.get('/sendgcm', function(req, res) {
		res.render('pages/sendgcm');
	});

	// PROFILE SECTION =========================
	app.get('/profile', isLoggedIn, function(req, res) {
		res.render('pages/auth/profile', {
			user : req.user
		});
	});

	// LOGOUT ==============================
	app.get('/logout', function(req, res) {
		req.logout();
		res.redirect('/');
	});

	app.get('/api/todos', api.listar_notas);
	app.post('/api/todos', api.anotar);
	app.delete('/api/todos/:todo_id', api.borrar_nota);    
	app.get('/todo', function(req, res) {
	    res.sendfile('pages/todo'); // load the single view file (angular will handle the page changes on the front-end)
	});
	// application -------------------------------------------------------------
/*	app.get('*', function(req, res) {
	    res.sendfile('./public/index.html', { root: __dirname }); // load the single view file (angular will handle the page changes on the front-end)
	});*/
// =============================================================================
// AUTHENTICATE (FIRST LOGIN) ==================================================
// =============================================================================

	// locally --------------------------------
	// LOGIN ===============================
	// show the login form
	app.get('/login', function(req, res) {
		res.render('pages/auth/login', { message: req.flash('loginMessage') });
	});

	// process the login form
	app.post('/login', passport.authenticate('local-login', {
		successRedirect : '/dashboard', // redirect to the secure profile section
		failureRedirect : '/login', // redirect back to the signup page if there is an error
		failureFlash : true // allow flash messages
	}));

	// process the login form
	app.post('/api/login', usuario.localLogin);
	// SIGNUP =================================
	// show the signup form
	app.get('/signup', function(req, res) {
		res.render('pages/auth/signup', { message: req.flash('loginMessage') });
	});

	// process the signup form
	app.post('/signup', passport.authenticate('local-signup', {
		successRedirect : '/dashboard', // redirect to the secure profile section
		failureRedirect : '/signup', // redirect back to the signup page if there is an error
		failureFlash : true // allow flash messages
	}));

	// google ---------------------------------

	// send to google to do the authentication
	app.get('/auth/google', passport.authenticate('google', { scope : ['profile', 'email'] }));

	// the callback after google has authenticated the user
	app.get('/auth/google/callback',
		passport.authenticate('google', {
			successRedirect : '/dashboard',
			failureRedirect : '/'
		}));

// =============================================================================
// AUTHORIZE (ALREADY LOGGED IN / CONNECTING OTHER SOCIAL ACCOUNT) =============
// =============================================================================

    // locally --------------------------------
        app.get('/connect/local', function(req, res) {
            res.render('pages/auth/connect-local', { message: req.flash('loginMessage') });
        });
        app.post('/connect/local', passport.authenticate('local-signup', {
            successRedirect : '/dashboard', // redirect to the secure profile section
            failureRedirect : '/connect/local', // redirect back to the signup page if there is an error
            failureFlash : true // allow flash messages
        }));

    // google ---------------------------------

        // send to google to do the authentication
        app.get('/connect/google', passport.authorize('google', { scope : ['profile', 'email'] }));

        // the callback after google has authorized the user
        app.get('/connect/google/callback',
            passport.authorize('google', {
                successRedirect : '/dashboard',
                failureRedirect : '/'
            }));

    // =============================================================================
	// UNLINK ACCOUNTS =============================================================
	// =============================================================================
	// used to unlink accounts. for social accounts, just remove the token
	// for local account, remove email and password
	// user account will stay active in case they want to reconnect in the future

	// local -----------------------------------
	app.get('/unlink/local', function(req, res) {
		var user            = req.user;
		user.local.email    = undefined;
		user.local.password = undefined;
		user.save(function(err) {
			res.redirect('/profile');
		});
	});

	// google ---------------------------------
	app.get('/unlink/google', function(req, res) {
		var user          = req.user;
		user.google.token = undefined;
		user.save(function(err) {
			res.redirect('/profile');
		});
	});

}

exports.setup = setup;

// route middleware to ensure user is logged in
function isLoggedIn(req, res, next) {
    if (req.isAuthenticated())
        return next();

    res.redirect('/');
}