// server.js

// set up ========================
var express  = require('express');
var http = require('http');
var appX      = express();                               // create our app w/ express
var mongoose = require('mongoose');                     // mongoose for mongodb
var morgan = require('morgan');             // log requests to the console (express4)
var bodyParser = require('body-parser');    // pull information from HTML POST (express4)
var methodOverride = require('method-override'); // simulate DELETE and PUT (express4)
var config = require('./config');
var routes = require('./routes/routes');
var fs = require('fs');
var errorHandler = require('errorhandler');
var cookieParser = require('cookie-parser');
var session = require('express-session');

var passport = require('passport');
var flash    = require('connect-flash');
var waterfall = require('async-waterfall');

require('./config/passport')(passport); // pass passport for configuration
//var reload = require('reload');

// configuration =================
mongoose.connect(config.db.heroku);
// define model =================

//mongoose.connect(config.db.test);
mongoose.set('debug', true);

// set the view engine to ejs
appX.set('view engine', 'ejs');
//var expressLogFile = fs.createWriteStream('./logs/express.log', {flags: 'a'});
appX.set('port', (process.env.PORT || 5000));
//app.use(morgan('combined', {stream: expressLogFile}));
appX.use(express.static(__dirname + '/public'));                 // set the static files location /public/img will be /img for users
appX.use(morgan('dev'));                                         // log every request to the console
appX.use(bodyParser.urlencoded({'extended':'true'}));            // parse application/x-www-form-urlencoded
appX.use(bodyParser.json());                                     // parse application/json
appX.use(bodyParser.json({ type: 'application/vnd.api+json' })); // parse application/vnd.api+json as json
appX.use(methodOverride());

// required for passport
appX.use(cookieParser('secret'));
appX.use(session({cookie: { maxAge: 60000 }}));
appX.use(passport.initialize());
appX.use(passport.session()); // persistent login sessions
appX.use(flash()); // use connect-flash for flash messages stored in session

// development only
if ('development' == appX.get('env')) {
  appX.use(errorHandler({ dumpExceptions: true, showStack: true }));
}
if ('production' == appX.get('env')) {
  appX.use(errorHandler());
}

// routes ======================================================================
routes.setup(appX, passport);
// api ---------------------------------------------------------------------

//var server = http.createServer(appX);
 
//reload code here  
//reload(server, appX);

// listen (start app with node server.js) ======================================
//server.listen(appX.get('port'), function() {
appX.listen(appX.get('port'), function() {
  console.log('Node app is running on port', appX.get('port'));
});
